# frozen_string_literal: true

require 'clamp'
require 'json'
require 'zn/dataset'
require 'zn/search_assembly'
require 'zn/search_assembler'
require 'zn/repos/json_repo'
require 'zn/associators/has_many'
require 'zn/associators/belongs_to'

module Zn
  # Cli composes a default search assembly and provides a CLI
  class Cli < Clamp::Command
    def search_assembly(config_file)
      assembly = Zn::SearchAssembly.new
      assembler = Zn::SearchAssembler.new(assembly)
      assembler.register_repo_class(:json, Zn::Repos::JsonRepo)
      assembler.register_associator_class(:has_many, Zn::Associators::HasMany)
      assembler.register_associator_class(:belongs_to, Zn::Associators::BelongsTo)
      assembler.assemble(config_file)
      assembly
    end

    subcommand 'search', 'Search the text in values of the given key' do
      option ['--exact', '-e'], :flag, 'Search for exact value'
      option ['--dataset', '-d'], 'DATASET', 'Dataset to search', required: true
      option ['--key', '-k'], 'KEY', 'Key to search under', required: true
      option ['--value', '-v'], 'VALUE', 'Search value', required: true
      option ['--config-file', '-c'], 'CONFIG_FILE', 'Config that defines datasets',
             required: true, environment_variable: 'ZN_CONFIG_FILE'

      def execute
        search_assembly(config_file).search(dataset: dataset, key: key, value: value, exact: exact?) do |entity|
          puts entity.to_json
        end
      end
    end

    subcommand 'associate', 'Find associated entities' do
      option ['--dataset', '-d'], 'DATASET', 'Dataset to associate from', required: true
      option ['--config-file', '-c'], 'CONFIG_FILE', 'Config that defines datasets',
             required: true, environment_variable: 'ZN_CONFIG_FILE'

      def execute
        entities = STDIN.map do |line|
          JSON.parse(line)
        end

        search_assembly(config_file).associate(entities, dataset: dataset) do |entity|
          puts entity.to_json
        end
      end
    end

    subcommand 'list-keys', 'List keys in a dataset' do
      option ['--dataset', '-d'], 'DATASET', 'Dataset to search', required: true
      option ['--config-file', '-c'], 'CONFIG_FILE', 'Config that defines datasets',
             required: true, environment_variable: 'ZN_CONFIG_FILE'

      def execute
        puts search_assembly(config_file).dataset_keys(dataset: dataset)
      end
    end

    subcommand 'list-datasets', 'List known datasets' do
      option ['--config-file', '-c'], 'CONFIG_FILE', 'Config that defines datasets',
             required: true, environment_variable: 'ZN_CONFIG_FILE'

      def execute
        puts search_assembly(config_file).dataset_names
      end
    end

    subcommand 'prettify-json', 'Pretty print JSON' do
      def execute
        STDIN.each do |line|
          puts JSON.pretty_generate(JSON.parse(line))
        end
      end
    end
  end
end
